/*
 * FADE - Secure Overlay Cloud Storage with File Assured Deletion
 * Copyright (C) 2010  Yang Tang
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
 * @file ClientConfig.cc
 * @author Yang Tang
 * @brief Client configuration file parser implementation
 */

#include <cstring>
#include <cstdio>
#include "ClientConfig.h"
#include "Client.h"
#include "../common/Panic.h"

static const int BUFFER_LENGTH = 8192;

void ClientConfig::startElement(void *data, const char *el, const char **attr)
{
	if (strcmp(el, "keymanager") == 0)
	{
		for (int i = 0; attr[i]; i += 2)
		{
			if (strcmp(attr[i], "address") == 0)
			{
				Client::instance()->setEphemerizerAddress(attr[i + 1]);
			}
		}
	}
	else if (strcmp(el, "cache") == 0)
	{
		for (int i = 0; attr[i]; i += 2)
		{
			if (strcmp(attr[i], "path") == 0)
			{
				Client::instance()->setPath(attr[i + 1]);
			}
		}
	}
	else if (strcmp(el, "local") == 0)
	{
		for (int i = 0; attr[i]; i += 2)
		{
			if (strcmp(attr[i], "path") == 0)
			{
				Client::instance()->addLocalStorage(attr[i + 1]);
			}
		}
	}
	else if (strcmp(el, "s3") == 0)
	{
		for (int i = 0; attr[i]; i += 2)
		{
			if (strcmp(attr[i], "bucket") == 0)
			{
				Client::instance()->addS3Storage(attr[i + 1]);
			}
		}
	}
	else if (strcmp(el, "secret") == 0)
	{
		for (int i = 0; attr[i]; i += 2)
		{
			if (strcmp(attr[i], "file") == 0)
			{
				Client::instance()->setSecretFile(attr[i + 1]);
			}
		}
	}
}

void ClientConfig::endElement(void *data, const char *el)
{
}

void ClientConfig::parse(const char *filename)
{
	XML_Parser parser = XML_ParserCreate(NULL);
	if (!parser)
	{
		Panic::instance()->die("Cannot allocate memory for XML parser.\n");
	}
	XML_SetElementHandler(parser, ClientConfig::startElement, ClientConfig::endElement);
	char buf[BUFFER_LENGTH];
	FILE *fp = fopen(filename, "r");
	for (;;)
	{
		int len = fread(buf, 1, BUFFER_LENGTH, fp);
		if (ferror(fp))
		{
			Panic::instance()->die("Error when reading the configuration file.\n");
		}
		bool done = feof(fp);
		if (XML_Parse(parser, buf, len, done) == XML_STATUS_ERROR)
		{
			Panic::instance()->die("Parse error at line %" XML_FMT_INT_MOD "u:\n%s\n",
					       XML_GetCurrentLineNumber(parser),
					       XML_ErrorString(XML_GetErrorCode(parser)));
		}
		if (done)
		{
			break;
		}
	}
	fclose(fp);
	XML_ParserFree(parser);
}
