/*
 * FADE - Secure Overlay Cloud Storage with File Assured Deletion
 * Copyright (C) 2010  Yang Tang
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
 * @file client/Main.cc
 * @author Yang Tang
 * @brief Main program for FADE client
 */

#include <cstdio>
#include <cstring>
#include "Client.h"
#include "../common/Panic.h"
#include "../common/Common.h"

#ifdef FADE_EVAL
#define TIME_DIFFERENCE(time1, time0) (difftime((time1).time, (time0).time) + 0.001 * ((time1).millitm - (time0).millitm))
#endif /* FADE_EVAL */

int main(int argc, const char *argv[])
{
	if (argc != 2)
	{
		Panic::instance()->die("usage: %s config_file\n", argv[0]);
	}
	Client::instance()->readConfig(argv[1]);
	Client::instance()->readSecret();
	for (;;)
	{
		fputs("FADE>> ", stdout);
		char command[COMMAND_BUFFER_LENGTH];
		fgets(command, COMMAND_BUFFER_LENGTH, stdin);
		const char *succeeded = "succeeded";
		const char *failed = "failed";
		const char *sep = " \t\n\r";
		char *operation = strtok(command, sep);
		char *filename = strtok(NULL, sep);
		char *token = strtok(NULL, sep);
		char *policy = NULL;
		if (token && strcasecmp(token, "POLICY") == 0)
		{
			policy = strtok(NULL, sep);
		}
		if (strcasecmp(operation, "UPLOAD") == 0)
		{
			if (!filename || !policy)
			{
				fputs("Usage: UPLOAD filename POLICY policy\n", stdout);
				continue;
			}
#ifdef FADE_EVAL
			ftime(&Client::instance()->allStartTime);
#endif /* FADE_EVAL */
			printf("UPLOAD %s.\n", Client::instance()->upload(filename, policy) ? succeeded : failed);
#ifdef FADE_EVAL
			ftime(&Client::instance()->allEndTime);
			printf("ALL\tFILE\tMETA\tENC\n%.3f\t%.3f\t%.3f\t%.3f\n", TIME_DIFFERENCE(Client::instance()->allEndTime, Client::instance()->allStartTime), TIME_DIFFERENCE(Client::instance()->fileEndTime, Client::instance()->fileStartTime), TIME_DIFFERENCE(Client::instance()->metaEndTime, Client::instance()->metaStartTime), TIME_DIFFERENCE(Client::instance()->encEndTime, Client::instance()->encStartTime));
#endif /* FADE_EVAL */
		}
		else if (strcasecmp(operation, "DOWNLOAD") == 0)
		{
			if (!filename)
			{
				fputs("Usage: DOWNLOAD filename\n", stdout);
				continue;
			}
#ifdef FADE_EVAL
			ftime(&Client::instance()->allStartTime);
#endif /* FADE_EVAL */
			printf("DOWNLOAD %s.\n", Client::instance()->download(filename) ? succeeded : failed);
#ifdef FADE_EVAL
			ftime(&Client::instance()->allEndTime);
			printf("ALL\tFILE\tMETA\tENC\n%.3f\t%.3f\t%.3f\t%.3f\n", TIME_DIFFERENCE(Client::instance()->allEndTime, Client::instance()->allStartTime), TIME_DIFFERENCE(Client::instance()->fileEndTime, Client::instance()->fileStartTime), TIME_DIFFERENCE(Client::instance()->metaEndTime, Client::instance()->metaStartTime), TIME_DIFFERENCE(Client::instance()->encEndTime, Client::instance()->encStartTime) + TIME_DIFFERENCE(Client::instance()->hmacEndTime, Client::instance()->hmacStartTime));
#endif /* FADE_EVAL */
		}
		else if (strcasecmp(operation, "RENEW") == 0)
		{
			if (!filename || !policy)
			{
				fputs("Usage: RENEW filename POLICY new_policy\n", stdout);
				continue;
			}
#ifdef FADE_EVAL
			ftime(&Client::instance()->allStartTime);
#endif /* FADE_EVAL */
			printf("RENEW %s.\n", Client::instance()->renew(filename, policy) ? succeeded : failed);
#ifdef FADE_EVAL
			ftime(&Client::instance()->allEndTime);
			printf("ALL\tMETA_DN\tMETA_UP\n%.3f\t%.3f\t%.3f\n", TIME_DIFFERENCE(Client::instance()->allEndTime, Client::instance()->allStartTime), TIME_DIFFERENCE(Client::instance()->metaEndTime, Client::instance()->metaStartTime), TIME_DIFFERENCE(Client::instance()->metaEndTime2, Client::instance()->metaStartTime2));
#endif /* FADE_EVAL */
		}
		else if (strcasecmp(operation, "GENSECRET") == 0)
		{
			printf("GENSECRET %s.\n", Client::instance()->generateSecret() ? succeeded : failed);
		}
		else if (strcasecmp(operation, "QUIT") == 0)
		{
			break;
		}
	}
	return 0;
}
