/*
 * FADE - Secure Overlay Cloud Storage with File Assured Deletion
 * Copyright (C) 2010  Yang Tang
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
 * @file Storage.h
 * @author Yang Tang
 * @brief Storage abstract interface
 */

#ifndef FADE_STORAGE_H
#define FADE_STORAGE_H

#include <string>

using namespace std;

/**
 * @brief Storage abstract interface
 */
class Storage
{
public:
	/**
	 * Put a file to the storage
	 * @param filename The filename
	 * @return True if successful, otherwise false
	 */
	virtual bool put(const string &filename) const = 0;

	/**
	 * Get a file from the storage
	 * @param filename The filename
	 * @return True if successful, otherwise false
	 */
	virtual bool get(const string &filename) const = 0;
};

#endif /* FADE_STORAGE_H */
