/*
 * FADE - Secure Overlay Cloud Storage with File Assured Deletion
 * Copyright (C) 2010  Yang Tang
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
 * @file Panic.cc
 * @author Yang Tang
 * @brief Panic thrower implementation
 */

#include <cstdio>
#include <cstdarg>
#include <cstdlib>
#include "Panic.h"

Panic *Panic::instance()
{
	static Panic _instance;
	return &_instance;
}

void Panic::die(const char *format, ...) const
{
	if (format)
	{
		va_list args;
		va_start(args, format);
		vfprintf(stderr, format, args);
		va_end(args);
	}
	exit(-1);
}

void Panic::warn(const char *format, ...) const
{
	if (format)
	{
		va_list args;
		va_start(args, format);
		vfprintf(stderr, format, args);
		va_end(args);
	}
}
