/*
 * FADE - Secure Overlay Cloud Storage with File Assured Deletion
 * Copyright (C) 2010  Yang Tang
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
 * @file PolicyForEphemerizer.cc
 * @author Yang Tang
 * @brief Policy implementation for FADE key manager
 */

#include <cstdio>
#include <openssl/pem.h>
#include "PolicyForEphemerizer.h"
#include "../common/Panic.h"

PolicyForEphemerizer::PolicyForEphemerizer(const string &policyName)
	: policyName(policyName)
{
	if (exist(policyName))
	{
		readPEM();
	}
	else
	{
		generateKey();
		writePEM();
	}
	ctx = BN_CTX_new();
}

void PolicyForEphemerizer::generateKey()
{
	rsa = RSA_generate_key(1024, 65537, NULL, NULL);
}

void PolicyForEphemerizer::readPEM()
{
	FILE *fp = fopen(("keys/" + policyName + ".pem").c_str(), "r");
	rsa = PEM_read_RSAPrivateKey(fp, NULL, NULL, NULL);
	if (!rsa)
	{
		Panic::instance()->warn("Error reading PEM for policy %s\n", policyName.c_str());
	}
	fclose(fp);
}

void PolicyForEphemerizer::writePEM() const
{
	FILE *fp = fopen(("keys/" + policyName + ".pem").c_str(), "w");
	if (!PEM_write_RSAPrivateKey(fp, rsa, NULL, NULL, 0, NULL, NULL))
	{
		Panic::instance()->warn("Error writing PEM for policy %s\n", policyName.c_str());
	}
	fclose(fp);
}

int PolicyForEphemerizer::decrypt(int flen, const unsigned char *from, unsigned char *to) const
{
	BIGNUM *ke = BN_bin2bn(from, flen, NULL);
	BIGNUM *k = BN_new();
	BN_mod_exp(k, ke, rsa->d, rsa->n, ctx);
	BN_free(ke);
	const int len = BN_bn2bin(k, to);
	BN_free(k);
	return len;
}
